@@ -7,7 +7,7 @@
  *
  *              More mods Robert Bond, 12/86
  *
- *		$Revision: 7.1 $
+ *		$Revision: 7.2 $
  */
 
 #include <sys/types.h>
@@ -112,8 +112,8 @@ deleterow(register int arg)
     int rs = maxrow - currow + 1;
     char buf[50];
 
-    if (rs - arg < 0)
-    {	rs = rs > 0 ? rs : 0;
+    if (rs - arg < 0) {
+	rs = rs > 0 ? rs : 0;
 	(void) sprintf(buf, "Can't delete %d row%s %d row%s left", arg,
 		(arg != 1 ? "s," : ","), rs, (rs != 1 ? "s" : ""));
 	error(buf);
@@ -250,10 +250,10 @@ pullcells(int to_insert)
     modflg++;
 
     for (p = to_fix; p; p = p->next) {
-	n = lookat (p->row + deltar, p->col + deltac);
+	n = lookat(p->row + deltar, p->col + deltac);
 	(void) clearent(n);
-	copyent( n, p, deltar, deltac);
-	n -> flags = p -> flags & ~is_deleted;
+	copyent(n, p, deltar, deltac);
+	n->flags = p->flags & ~is_deleted;
     }
 }
 
@@ -374,6 +374,8 @@ openrow(int rs)
     for (i=0; i < 26; i++) {	/* update all marked cells */
 	if (savedrow[i] >= rs)
 	    savedrow[i]++;
+	if (savedstrow[i] >= rs)
+	    savedstrow[i]++;
     }
     FullUpdate++;
     modflg++;
@@ -422,6 +424,8 @@ closerow(register r)
 	}
 	if (savedrow[i] > r)
 	    savedrow[i]--;
+	if (savedstrow[i] > r)
+	    savedstrow[i]--;
     }
 
     maxrow--;
@@ -470,6 +474,8 @@ opencol(int cs, int numcol)
     for (i=0; i < 26; i++) {	/* update all marked cells */
 	if (savedcol[i] >= cs)
 	    savedcol[i] += numcol;
+	if (savedstcol[i] >= cs)
+	    savedstcol[i] += numcol;
     }
     FullUpdate++;
     modflg++;
@@ -541,8 +547,13 @@ closecol(int cs, int numcol)
 	    savedrow[i] = -1;
 	    savedcol[i] = -1;
 	}
+	if (savedstcol[i] >= cs && savedstcol[i] < cs+numcol) {
+	    savedstcol[i] = cs;
+	}
 	if (savedcol[i] > cs)
 	    savedcol[i] -= numcol;
+	if (savedstcol[i] > cs)
+	    savedstcol[i] -= numcol;
     }
 
     maxcol -= numcol;
@@ -1443,9 +1454,10 @@ write_fd(register FILE *f, int r0, int c0, int rn, int cn)
 						     (*pp)->row) ;
 	    }
     }
-    if (rndinfinity)
-	fprintf(f, "set rndinfinity\n");
-    fprintf(f, "goto %s\n", v_name( currow, curcol ) );
+    if (rndtoeven)
+	fprintf(f, "set rndtoeven\n");
+    fprintf(f, "goto %s", v_name(currow, curcol));
+    fprintf(f, " %s\n", v_name(strow, stcol));
 }
 
 int
@@ -1568,7 +1580,7 @@ erasedb()
 void
 backcol(int arg)
 {
-    while (--arg>=0) {
+    while (--arg >= 0) {
 	if (curcol)
 	    curcol--;
 	else
@@ -1576,6 +1588,8 @@ backcol(int arg)
 	while(col_hidden[curcol] && curcol)
 	    curcol--;
     }
+    rowsinrange = 1;
+    colsinrange = fwidth[curcol];
 }
 
 /* moves curcol forward one displayed column */
@@ -1593,6 +1607,8 @@ forwcol(int arg)
 	while (col_hidden[curcol]&&(curcol<maxcols-1))
 	    curcol++;
     }
+    rowsinrange = 1;
+    colsinrange = fwidth[curcol];
 }
 
 /* moves currow forward one displayed row */
@@ -1610,6 +1626,8 @@ forwrow(int arg)
 	while (row_hidden[currow]&&(currow<maxrows-1))
 	    currow++;
     }
+    rowsinrange = 1;
+    colsinrange = fwidth[curcol];
 }
 
 /* moves currow backward one displayed row */
@@ -1624,6 +1642,8 @@ backrow(int arg)
 	while (row_hidden[currow] && currow)
 	    currow--;
     }
+    rowsinrange = 1;
+    colsinrange = fwidth[curcol];
 }
 
 
