@@ -2,13 +2,13 @@
  * Help functions for sc 
  * R. Bond, 1988
  * J. Buhrt 1990
- * $Revision: 7.1 $
+ * $Revision: 7.2 $
  */
 
 #ifdef QREF
 #include <stdio.h>
 char	*header = " Quick Reference";
-char	*revision = "$Revision: 7.1 $";
+char	*revision = "$Revision: 7.2 $";
 #else
 #include <curses.h>
 #include "sc.h"
@@ -95,10 +95,11 @@ char *setoptions[] = {
 "          iterations=n  Set the number of iterations allowed. (10)",
 "          tblstyle=xx   Set ``T'' output style to:",
 "                        0 (none), tex, latex, slatex, or tbl.",
-"          rndinfinity   Round to infinity (round .5 up vs to nearest even).",
+"          rndtoeven     Round *.5 to nearest even number instead of"
+"                        always up.",
 "          rowlimit=n    Set the remembered row limit for newline action.",
 "          collimit=n    Set the remembered column limit for newline action.",
-"                     (rowlimit and collimit can both be set by ^Tz)",
+"                        (rowlimit and collimit can both be set by ^Tz)",
 (char *)0
 };
 
