@@ -7,7 +7,7 @@
  *
  *              More mods Robert Bond, 12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
- *		$Revision: 7.1 $
+ *		$Revision: 7.2 $
  */
 
 #include <sys/types.h>
@@ -56,18 +56,11 @@ char *regex();
     int doquit();
 #endif
 
-/* Use this structure to save the the last 'g' command */
-struct go_save {
-	int g_type;
-	double g_n;
-	char *g_s;
-	int  g_row;
-	int  g_col;
-	int  errsearch;
-} gs;
+/* Use this structure to save the last 'g' command */
+struct go_save gs;
 
 /* g_type can be: */
-#define G_NONE 0			/* Starting value - must be 0*/
+#define G_NONE 0			/* Starting value - must be 0 */
 #define G_NUM 1
 #define G_STR 2
 #define G_CELL 3
@@ -454,7 +447,7 @@ dodts(int e1, int e2, int e3)
     time_t	secs;
     struct tm	t; 
 
-    if (e1 > 0 && e1 < 13 && e3 > 31) {
+    if (e2 > 12 || e3 > 31) {
 	mo  = e1;
 	day = e2;
 	yr  = e3;
@@ -576,7 +569,7 @@ doeqs(char *s1, char *s2)
  */
 
 struct ent *
-getent (char *colstr, double rowdoub)
+getent(char *colstr, double rowdoub)
 {
     int collen;		/* length of string */
     int row, col;	/* integer values   */
@@ -608,7 +601,7 @@ getent (char *colstr, double rowdoub)
  */
 
 double
-donval (char *colstr, double rowdoub)
+donval(char *colstr, double rowdoub)
 {
     struct ent *ep;
 
@@ -781,31 +774,30 @@ eval(register struct enode *e)
 	case DTR:	 return (dtr(eval(e->e.o.right)));
 	case RTD:	 return (rtd(eval(e->e.o.right)));
 	case RND:
-		if (rndinfinity)
-		{	double temp = eval(e->e.o.right);
-			return(temp-floor(temp) < 0.5 ?
-					floor(temp) : ceil(temp));
-		}
-		else
-			return rint(eval(e->e.o.right));
+	    if (rndtoeven)
+		return rint(eval(e->e.o.right));
+	    else {
+		double temp = eval(e->e.o.right);
+		return(temp-floor(temp) < 0.5 ? floor(temp) : ceil(temp));
+	    }
  	case ROUND:
-		{	int prec = (int) eval(e->e.o.right);
-			double	scal = 1;
-			if (0 < prec)
-				do scal *= 10; while (0 < --prec);
-			else if (prec < 0)
-				do scal /= 10; while (++prec < 0);
+	    {	int prec = (int) eval(e->e.o.right);
+		double	scal = 1;
+		if (0 < prec)
+		    do scal *= 10; while (0 < --prec);
+		else if (prec < 0)
+		    do scal /= 10; while (++prec < 0);
 
-			if (rndinfinity)
-			{	double temp = eval(e->e.o.left);
-				temp *= scal;
-				temp = ((temp-floor(temp)) < 0.5 ?
-						floor(temp) : ceil(temp));
-				return(temp / scal);
-			}
-			else
-				return(rint(eval(e->e.o.left) * scal) / scal);
+		if (rndtoeven)
+		    return (rint(eval(e->e.o.left) * scal) / scal);
+		else {
+		    double temp = eval(e->e.o.left);
+		    temp *= scal;
+		    temp = ((temp-floor(temp)) < 0.5 ?
+			    floor(temp) : ceil(temp));
+		    return(temp / scal);
 		}
+	    }
 	case FV:
 	case PV:
 	case PMT:	return(finfunc(e->op,eval(e->e.o.left),
@@ -964,7 +956,7 @@ doext(char *command, double value)
 #else /* VMS */
 
 char *
-doext (char *command, double value)
+doext(char *command, double value)
 {
     static char *prevstr = (char *)0;	/* previous result */
     static unsigned	prevlen = 0;
@@ -990,20 +982,20 @@ doext (char *command, double value)
 	    error ("Running external function...");
 	    (void) refresh();
 
-	    if ((pp = popen (buff, "r")) == (FILE *) NULL) {	/* run it */
+	    if ((pp = popen(buff, "r")) == (FILE *) NULL) {	/* run it */
 		error ("Warning: running \"%s\" failed", buff);
 		cellerror = CELLERROR;
 	    }
 	    else {
-		if (fgets (buff, sizeof(buff)-1, pp) == NULL)	/* one line */
-		    error ("Warning: external function returned nothing");
+		if (fgets(buff, sizeof(buff)-1, pp) == NULL)	/* one line */
+		    error("Warning: external function returned nothing");
 		else {
 		    char *cp;
 
-		    error ("");				/* erase notice */
+		    error("");				/* erase notice */
 		    buff[sizeof(buff)-1] = '\0';
 
-		    if (cp = strchr (buff, '\n'))	/* contains newline */
+		    if (cp = strchr(buff, '\n'))	/* contains newline */
 			*cp = '\0';			/* end string there */
 
 		    if (strlen(buff) + 1 > prevlen)
@@ -1035,7 +1027,7 @@ doext (char *command, double value)
  */
 
 char *
-dosval (char *colstr, double rowdoub)
+dosval(char *colstr, double rowdoub)
 {
     struct ent *ep;
     char *llabel;
@@ -1213,11 +1205,11 @@ int repct = 1;		/* Make repct a global variable so that the
 void
 setiterations(int i)
 {
-	if(i<1) {
-		error("iteration count must be at least 1");
-		propagation = 1;
-		}
-	else propagation = i;
+    if (i < 1) {
+	error("iteration count must be at least 1");
+	propagation = 1;
+	}
+    else propagation = i;
 }
 
 void
@@ -1229,8 +1221,8 @@ EvalAll()
     (void) signal(SIGFPE, eval_fpe);
 
     while ((lastcnt = RealEvalAll()) && (++repct <= propagation));
-    if((propagation>1)&& (lastcnt >0 ))
- 	error("Still changing after %d iterations",propagation-1);
+    if ((propagation > 1) && (lastcnt > 0))
+ 	error("Still changing after %d iterations",repct-1);
 
     (void) signal(SIGFPE, doquit);
 }
@@ -1266,46 +1258,53 @@ RealEvalAll()
 void
 RealEvalOne(register struct ent *p, int i, int j, int *chgct)
 {
-	if (p->flags & is_strexpr) {
-	    char *v;
-	    if (setjmp(fpe_save)) {
-		error("Floating point exception %s", v_name(i, j));
-		cellerror = CELLERROR;
-		v = "";
-	    } else {
-		cellerror = CELLOK;
-		v = seval(p->expr);
-	    }
-	    p->cellerror = cellerror;
-	    if (!v && !p->label) /* Everything's fine */
-		return;
-	    if (!p->label || !v || strcmp(v, p->label) != 0 || cellerror) {
-		(*chgct)++;
-		p->flags |= is_changed;
-		changed++;
-	    }
-	    if(p->label)
-		scxfree(p->label);
-	    p->label = v;
+    if (p->flags & is_strexpr) {
+	char *v;
+	if (setjmp(fpe_save)) {
+	    error("Floating point exception %s", v_name(i, j));
+	    cellerror = CELLERROR;
+	    v = "";
 	} else {
-	    double v;
-	    if (setjmp(fpe_save)) {
-		error("Floating point exception %s", v_name(i, j));
-		cellerror = CELLERROR;
-		v = (double)0.0;
-	    } else {
-		cellerror = CELLOK;
-		gmyrow=i; gmycol=j;
-		v = eval (p->expr);
-	    }
-	    if ((p->cellerror = cellerror) || (v != p->v)) {
-		p->v = v;
-		if (!cellerror)		/* don't keep eval'ing a error */
-			(*chgct)++;
-		p->flags |= is_changed|is_valid;
-		changed++;
-	    }
+	    cellerror = CELLOK;
+	    v = seval(p->expr);
 	}
+	p->cellerror = cellerror;
+	if (!v && !p->label) /* Everything's fine */
+	    return;
+	if (!p->label || !v || strcmp(v, p->label) != 0 || cellerror) {
+	    (*chgct)++;
+	    p->flags |= is_changed;
+	    changed++;
+	}
+	if(p->label)
+	    scxfree(p->label);
+	p->label = v;
+    } else {
+	double v;
+	if (setjmp(fpe_save)) {
+	    error("Floating point exception %s", v_name(i, j));
+	    cellerror = CELLERROR;
+	    v = (double)0.0;
+	} else {
+	    cellerror = CELLOK;
+	    gmyrow=i; gmycol=j;
+	    v = eval(p->expr);
+	}
+	if ((p->cellerror = cellerror) || (v != p->v)) {
+		/*
+		 * The following line serves no useful purpose, but
+		 * without it, it seems to give the "Still changing
+		 * after x iterations" message a lot when it shouldn't.
+		 * See the BUGS file for more information. - CRM
+		 */
+	    fflush(stdout);
+	    p->v = v;
+	    if (!cellerror)	/* don't keep eval'ing an error */
+		(*chgct)++;
+	    p->flags |= is_changed|is_valid;
+	    changed++;
+	}
+    }
 }
 
 struct enode *
@@ -1486,7 +1485,8 @@ go_last()
 		num_search(gs.g_n, gs.errsearch);
 		break;
     case  G_CELL:
-		moveto(gs.g_row, gs.g_col);
+		moveto(gs.g_row, gs.g_col, gs.g_lastrow, gs.g_lastcol,
+		    gs.strow, gs.stcol);
 	    	break;
     case  G_STR: 
 		gs.g_type = G_NONE;	/* Don't free the string */
@@ -1497,16 +1497,38 @@ go_last()
     }
 }
 
-/* place the cursor on a given cell */
+/* Place the cursor on a given cell.  If cornerrow >= 0, place the cell
+   at row cornerrow and column cornercol in the upper left corner of the
+   screen if possible. */
 void
-moveto(int row, int col)
+moveto(int row, int col, int lastrow, int lastcol, int cornerrow, int cornercol)
 {
+    register int i;
     currow = row;
     curcol = col;
+    if (cornerrow >= 0) {
+	strow = cornerrow;
+	stcol = cornercol;
+    }
     g_free();
     gs.g_type = G_CELL;
     gs.g_row = currow;
     gs.g_col = curcol;
+    gs.g_lastrow = lastrow;
+    gs.g_lastcol = lastcol;
+    gs.strow = cornerrow;
+    gs.stcol = cornercol;
+    for (rowsinrange = 0, i = row; i <= lastrow; i++) {
+	if (row_hidden[i])
+	    continue;
+	rowsinrange++;
+    }
+    for (colsinrange = 0, i = col; i <= lastcol; i++) {
+	if (col_hidden[i])
+	    continue;
+	colsinrange += fwidth[i];
+    }
+    FullUpdate++;
 }
 
 /*
@@ -1539,7 +1561,7 @@ num_search(double n, int errsearch)
 	    c++;
 	else {
 	    if (r < maxrow) {
-		while(++r < maxrow && row_hidden[r]) /* */;
+		while (++r < maxrow && row_hidden[r]) /* */;
 		c = 0;
 	    } else {
 		r = 0;
@@ -1562,6 +1584,8 @@ num_search(double n, int errsearch)
 		
     currow = r;
     curcol = c;
+    rowsinrange = 1;
+    colsinrange = fwidth[curcol];
 }
 
 /* 'goto' a cell containing a matching string */
@@ -1606,7 +1630,7 @@ str_search(char *s)
 	    c++;
 	else {
 	    if (r < maxrow) {
-		while(++r < maxrow && row_hidden[r]) /* */;
+		while (++r < maxrow && row_hidden[r]) /* */;
 		c = 0;
 	    } else {
 		r = 0;
@@ -1621,7 +1645,7 @@ str_search(char *s)
 	    return;
 	}
 	p = *ATBL(tbl, r, c);
-    } while(col_hidden[c] || !p || !(p->label)
+    } while (col_hidden[c] || !p || !(p->label)
 #if defined(RE_COMP)
 		  			|| (re_exec(p->label) == 0));
 #else
@@ -1633,6 +1657,8 @@ str_search(char *s)
 #endif
     currow = r;
     curcol = c;
+    rowsinrange = 1;
+    colsinrange = fwidth[curcol];
 #if defined(REGCMP)
     free(tmp);
 #endif
@@ -1744,6 +1770,8 @@ let(struct ent *v, struct enode *e)
     double val;
     unsigned isconstant = constant(e);
 
+    if (v->row == currow && v->col == curcol)
+	cellassign = 1;
     if (loading && !isconstant)
 	val = (double)0.0;
     else
@@ -1806,6 +1834,7 @@ slet(struct ent *v, struct enode *se, int flushdir)
 {
     char *p;
 
+    cellassign = 1;
     exprerr = 0;
     (void) signal(SIGFPE, eval_fpe);
     if (setjmp(fpe_save)) {
@@ -2018,16 +2047,16 @@ decodev (struct ent_ptr v)
 {
 	register struct range *r;
 
-	if (!v.vp) (void) sprintf (line+linelim,"VAR?");
+	if (!v.vp) (void) sprintf(line+linelim,"VAR?");
 	else if ((r = find_range((char *)0, 0, v.vp, v.vp)) && !r->r_is_range)
 	    (void) sprintf(line+linelim, "%s", r->r_name);
 	else
-	    (void) sprintf (line+linelim, "%s%s%s%d",
-			v.vf & FIX_COL ? "$" : "",
-			coltoa(v.vp->col),
-			v.vf & FIX_ROW ? "$" : "",
-			v.vp->row);
-	linelim += strlen (line+linelim);
+	    (void) sprintf(line+linelim, "%s%s%s%d",
+		v.vf & FIX_COL ? "$" : "",
+		coltoa(v.vp->col),
+		v.vf & FIX_ROW ? "$" : "",
+		v.vp->row);
+	linelim += strlen(line+linelim);
 }
 
 char *
@@ -2080,20 +2109,20 @@ decompile(register struct enode *e, int	priority)
 	switch (e->op) {
 	case 'f':	for (s="fixed "; line[linelim++] = *s++;);
 			linelim--;
-			decompile (e->e.o.right, 30);
+			decompile(e->e.o.right, 30);
 			break;
 	case 'm':	line[linelim++] = '-';
-			decompile (e->e.o.right, 30);
+			decompile(e->e.o.right, 30);
 			break;
 	case '~':	line[linelim++] = '~';
-			decompile (e->e.o.right, 30);
+			decompile(e->e.o.right, 30);
 			break;
-	case O_VAR:	decodev (e->e.v);
+	case O_VAR:	decodev(e->e.v);
 			break;
-	case O_CONST:	(void) sprintf (line+linelim,"%.15g",e->e.k);
+	case O_CONST:	(void) sprintf(line+linelim,"%.15g",e->e.k);
 			linelim += strlen (line+linelim);
 			break;
-	case O_SCONST:	(void) sprintf (line+linelim, "\"%s\"", e->e.s);
+	case O_SCONST:	(void) sprintf(line+linelim, "\"%s\"", e->e.s);
 			linelim += strlen(line+linelim);
 			break;
 
@@ -2169,9 +2198,9 @@ decompile(register struct enode *e, int	priority)
 	case NUMITER:	for ( s = "@numiter"; line[linelim++] = *s++;);
                         linelim--;
                         break;
-	default:	decompile (e->e.o.left, mypriority);
+	default:	decompile(e->e.o.left, mypriority);
 			line[linelim++] = e->op;
-			decompile (e->e.o.right, mypriority+1);
+			decompile(e->e.o.right, mypriority+1);
 			break;
 	}
 	if (mypriority<priority) line[linelim++] = ')';
@@ -2257,9 +2286,9 @@ range_arg(char *s, struct enode *e)
 	(void) sprintf(line+linelim, "%s", r->r_name);
 	linelim += strlen(line+linelim);
     } else {
-	decodev (e->e.r.left);
+	decodev(e->e.r.left);
 	line[linelim++] = ':';
-	decodev (e->e.r.right);
+	decodev(e->e.r.right);
     }
     line[linelim++] = ')';
 }
@@ -2271,7 +2300,7 @@ editfmt(int row, int col)
 
     p = lookat (row, col);
     if (p->format) {
-        (void) sprintf (line, "fmt %s \"%s\"", v_name(row, col), p->format);
+        (void) sprintf(line, "fmt %s \"%s\"", v_name(row, col), p->format);
 	linelim = strlen(line);
     }
 }
@@ -2282,11 +2311,11 @@ editv(int row, int col)
     register struct ent *p;
 
     p = lookat (row, col);
-    (void) sprintf (line, "let %s = ", v_name(row, col));
+    (void) sprintf(line, "let %s = ", v_name(row, col));
     linelim = strlen(line);
     if (p->flags & is_strexpr || p->expr == 0) {
-	(void) sprintf (line+linelim, "%.15g", p->v);
-	linelim += strlen (line+linelim);
+	(void) sprintf(line+linelim, "%.15g", p->v);
+	linelim += strlen(line+linelim);
     } else {
         editexp(row,col);
     }
@@ -2307,21 +2336,21 @@ edits(int row, int col)
 {
     register struct ent *p;
 
-    p = lookat (row, col);
-    if( p->flags&is_label )
+    p = lookat(row, col);
+    if (p->flags&is_label)
 	(void) sprintf( line, "label %s = ", v_name(row, col));
     else
-    (void) sprintf (line, "%sstring %s = ",
-			((p->flags&is_leftflush) ? "left" : "right"),
-			v_name(row, col));
+    (void) sprintf(line, "%sstring %s = ",
+		((p->flags&is_leftflush) ? "left" : "right"),
+		v_name(row, col));
     linelim = strlen(line);
     if (p->flags & is_strexpr && p->expr) {
 	editexp(row, col);
     } else if (p->label) {
-        (void) sprintf (line+linelim, "\"%s\"", p->label);
+        (void) sprintf(line+linelim, "\"%s\"", p->label);
         linelim += strlen (line+linelim);
     } else {
-        (void) sprintf (line+linelim, "\"");
+        (void) sprintf(line+linelim, "\"");
         linelim += 1;
     }
 }
