@@ -7,8 +7,7 @@
  *
  *              More mods Robert Bond, 12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
- *		Currently supported by sequent!sawmill!buhrt (Jeff Buhrt)
- *		$Revision: 7.1 $
+ *		$Revision: 7.2 $
  *
  */
 
@@ -47,6 +46,7 @@ struct ent ***tbl;
 int strow = 0, stcol = 0;
 int currow = 0, curcol = 0;
 int savedrow[26], savedcol[26];
+int savedstrow[26], savedstcol[26];
 int FullUpdate = 0;
 int ClearScreen = 0;	/* don't try to be smart */
 int maxrow, maxcol;
@@ -60,6 +60,7 @@ char line[FBUFLEN];
 int changed;
 struct ent *to_fix;
 int modflg;
+int cellassign;
 int numeric;
 char *mdir;
 int showsc, showsr;	/* Starting cell for highlighted range */
@@ -86,11 +87,13 @@ int  autolabel = 1;     /* If room, causes label to be created after a define*/
 int  calc_order = BYROWS;
 int  optimize = 0;	/* Causes numeric expressions to be optimized */
 int  tbl_style = 0;	/* headers for T command output */
-int  rndinfinity = 0;
+int  rndtoeven = 0;
 int  numeric_field = 0; /* Started the line editing with a number */
 int  craction = 0;	/* 1 for down, 2 for right */
 int  rowlimit = -1;
 int  collimit = -1;
+int  rowsinrange = 1;
+int  colsinrange = DEFWIDTH;
 #ifdef	SIGWINCH
 int  hitwinch = 0;	/* got a SIGWINCH? */
 #endif
@@ -246,6 +249,9 @@ main (int argc, char  **argv)
 	    case 'R':
 		    craction = CRROWS;
 		    break;
+	    case 'e':
+		    rndtoeven = 1;
+		    break;
 	    default:
 		    (void) fprintf(stderr,"%s: unrecognized option: \"%c\"\n",
 			progname,argv[0][1]);
@@ -295,6 +301,7 @@ main (int argc, char  **argv)
 	calc_order = BYROWS;
 
     modflg = 0;
+    cellassign = 0;
 #ifdef VENIX
     setbuf (stdin, NULL);
 #endif
@@ -305,22 +312,24 @@ main (int argc, char  **argv)
 	nedistate = -1;
 	narg = 1;
 	if (edistate < 0 && linelim < 0 && autocalc && (changed || FullUpdate))
-	{    EvalAll();
-	     if (changed)		/* if EvalAll changed or was before */
+	{
+	    EvalAll();
+	    if (changed)		/* if EvalAll changed or was before */
 		anychanged = TRUE;
-	     changed = 0;
+	    changed = 0;
 	}
 	else		/* any cells change? */
 	if (changed)
-	     anychanged = TRUE;
+	    anychanged = TRUE;
 
 #ifdef	SIGWINCH
 	/* got a SIGWINCH? */
 	if (hitwinch)
-	{	hitwinch = 0;
-		stopdisp();
-		startdisp();
-		FullUpdate++;
+	{
+	    hitwinch = 0;
+	    stopdisp();
+	    startdisp();
+	    FullUpdate++;
 	}
 #endif
 	update(anychanged);
@@ -375,10 +384,13 @@ main (int argc, char  **argv)
 		    error ("No such command (^%c)", c + 0100);
 		    break;
 		case ctl('b'):
+		    c = craction;
 		    if (numeric_field) {
+			craction = 0;
 			write_line(ctl('m'));
 			numeric_field = 0;
 		    }
+		    craction = c;
 		    backcol(arg);
 		    break;
 		case ctl('c'):
@@ -407,10 +419,13 @@ main (int argc, char  **argv)
 		    break;
 
 		case ctl('f'):
+		    c = craction;
 		    if (numeric_field) {
+			craction = 0;
 			write_line(ctl('m'));
 			numeric_field = 0;
 		    }
+		    craction = c;
 		    forwcol(arg);
 #ifdef RIGHT_CBUG
 		    wasforw++;
@@ -465,18 +480,24 @@ main (int argc, char  **argv)
 		    break;
 
 		case ctl('n'):
+		    c = craction;
 		    if (numeric_field) {
+			craction = 0;
 			write_line(ctl('m'));
 			numeric_field = 0;
 		    }
+		    craction = c;
 		    forwrow(arg);
 		    break;
 
 		case ctl('p'):
+		    c = craction;
 		    if (numeric_field) {
+			craction = 0;
 			write_line(ctl('m'));
 			numeric_field = 0;
 		    }
+		    craction = c;
 		    backrow(arg);
 		    break;
 
@@ -555,28 +576,30 @@ main (int argc, char  **argv)
 			    --modflg;	/* negate the modflg++ */
 			    break;
 			case 'r': case 'R':
-			    do {
-				++craction;
-				if (craction >= 3)
+			    error("Which direction after return key?");
+			    switch(nmgetch()) {
+				case ctl('m'):
 				    craction = 0;
-				switch(craction) {
-				    default:
-					craction = 0; /* fall through */
-				    case 0:
-					error("No action after new line"
-                                              " (press RETURN to accept)");
-					break;
-				    case CRROWS:
-					error("Down row after new line"
-                                              " (press RETURN to accept)");
-					break;
-				    case CRCOLS:
-					error("Right column after new line"
-                                              " (press RETURN to accept)");
-					break;
-				}
-			    } while (nmgetch() != '\r');
-			    error("");
+				    error("No action after new line");
+				    break;
+				case 'j':
+				case ctl('n'):
+				    craction = CRROWS;
+				    error("Down row after new line");
+				    break;
+				case 'l':
+				case ctl('f'):
+				case ' ':
+				    craction = CRCOLS;
+				    error("Right column after new line");
+				    break;
+				case ESC:
+				case ctl('g'):
+				    error("");
+				    break;
+				default:
+				    error("Not a valid direction");
+			    }
 			    break;
 			case 'z': case 'Z':
 			    rowlimit = currow;
@@ -644,9 +667,11 @@ main (int argc, char  **argv)
 	    /* we got a leading number */
 	    if (edistate != 0) {
 		/* First char of the count */
-		if (c == '0')      /* just a '0' goes to left col */
+		if (c == '0') {    /* just a '0' goes to left col */
 		    curcol = 0;
-		else {
+		    rowsinrange = 1;
+		    colsinrange = fwidth[curcol];
+		} else {
 		    nedistate = 0;
 		    narg = c - '0';
 		}
@@ -948,6 +973,8 @@ main (int argc, char  **argv)
 		    curcol = maxcols - 1;
 		    while (!VALID_CELL(p, currow, curcol) && curcol > 0)
 			curcol--;
+		    rowsinrange = 1;
+		    colsinrange = fwidth[curcol];
 		    break;
 		    }
 		case '#':
@@ -957,6 +984,8 @@ main (int argc, char  **argv)
 		    currow = maxrows - 1;
 		    while (!VALID_CELL(p, currow, curcol) && currow > 0)
 			currow--;
+		    rowsinrange = 1;
+		    colsinrange = fwidth[curcol];
 		    break;
 		    }
 		case 'w':
@@ -970,7 +999,8 @@ main (int argc, char  **argv)
 			    else {
 				if (currow < maxrows - 1) {
 				    while(++currow < maxrows - 1 &&
-					    row_hidden[currow]) /* */;
+					    row_hidden[currow])
+					;
 				    curcol = 0;
 				} else {
 				    error("At end of table");
@@ -980,6 +1010,8 @@ main (int argc, char  **argv)
 			} while(col_hidden[curcol] ||
 				!VALID_CELL(p, currow, curcol));
 		    }
+		    rowsinrange = 1;
+		    colsinrange = fwidth[curcol];
 		    break;
 		    }
 		case 'b':
@@ -992,8 +1024,8 @@ main (int argc, char  **argv)
 				curcol--;
 			    else {
 				if (currow) {
-				    while(--currow &&
-					row_hidden[currow]) /* */;
+				    while(--currow && row_hidden[currow])
+					;
 				    curcol = maxcols - 1;
 				} else {
 				    error ("At start of table");
@@ -1003,10 +1035,14 @@ main (int argc, char  **argv)
 			} while (col_hidden[curcol] ||
 				!VALID_CELL(p, currow, curcol));
 		    }
+		    rowsinrange = 1;
+		    colsinrange = fwidth[curcol];
 		    break;
 		    }
 		case '^':
 		    currow = 0;
+		    rowsinrange = 1;
+		    colsinrange = fwidth[curcol];
 		    break;
 		case '?':
 		    help();
@@ -1193,7 +1229,7 @@ main (int argc, char  **argv)
 			backcol((curcol-stcol+1)+1);
 			break;
 #ifdef KEY_NPAGE
-		case KEY_NPAGE:			/* page precedente */
+		case KEY_NPAGE:			/* next page */
 #endif
 		case 'J':
 		    forwrow((LINES-RESROW)/2);
@@ -1201,7 +1237,7 @@ main (int argc, char  **argv)
 		    FullUpdate++;
 		    break;
 #ifdef	KEY_PPAGE
-		case KEY_PPAGE:			/* page suivante */
+		case KEY_PPAGE:			/* previous page */
 #endif
 		case 'K':
 		    backrow((LINES-RESROW)/2);
@@ -1215,6 +1251,8 @@ main (int argc, char  **argv)
 		case KEY_HOME:
 			currow = 0;
 			curcol = 0;
+			rowsinrange = 1;
+			colsinrange = fwidth[curcol];
 			FullUpdate++;
 			break;
 #endif
@@ -1238,6 +1276,8 @@ main (int argc, char  **argv)
 		    error("");
 		    savedrow[c] = currow;
 		    savedcol[c] = curcol;
+		    savedstrow[c] = strow;
+		    savedstcol[c] = stcol;
 		    break;
 		case 'c':
 		    error("Copy marked cell (select a-z)");
@@ -1265,36 +1305,47 @@ main (int argc, char  **argv)
 			for (c1 = curcol; arg-- && c1 < maxcols; c1++) {
 			    n = lookat (currow, c1);
 			    (void) clearent(n);
-			    copyent( n, p, currow - savedrow[c], c1 - savedcol[c]);
+			    copyent(n, p, currow - savedrow[c], c1 - savedcol[c]);
 			}
 		    break;
 		    }
 		case '`': case '\'':
-		    error("Go to marked cell (select a-z)");
-		    if ((c=nmgetch()) == ESC || c == ctl('g')) {
+		    {
+			char c1 = c;
+			error("Go to marked cell (select a-z)");
+			if ((c = nmgetch()) == ESC || c == ctl('g')) {
+			    error("");
+			    break;
+			}
+			if ((c -= 'a') < 0 || c > 25) {
+			    error("Invalid character for mark (must be a-z)");
+			    break;
+			}
+			if (savedrow[c] == -1) {
+			    error("Mark not set");
+			    break;
+			}
 			error("");
+			currow = savedrow[c];
+			curcol = savedcol[c];
+			rowsinrange = 1;
+			colsinrange = fwidth[curcol];
+			if (c1 == '`') {
+			    strow = savedstrow[c];
+			    stcol = savedstcol[c];
+			}
+			FullUpdate++;
 			break;
 		    }
-		    if ((c -= 'a') < 0 || c > 25) {
-			error("Invalid character for mark (must be a-z)");
-			break;
-		    }
-		    if (savedrow[c] == -1) {
-			error("Mark not set");
-			break;
-		    }
-		    error("");
-		    currow = savedrow[c];
-		    curcol = savedcol[c];
-		    FullUpdate++;
-		    break;
 		case 'C':
+		    /* Force centering of current cell (or range, if we've
+		     * just jumped to a new range with the goto command).
+		     */
+		    strow = -1;
+		    stcol = -1;
 		    FullUpdate++;
 		    ClearScreen++;
 		    (void) clearok(stdscr,1);
-		    /* Center the display with cursor for middle */
-		    if(currow > (LINES-RESROW)/2)
-			strow = currow - ((LINES-RESROW)/2);
 		    break;
 		default:
 		    if ((toascii(c)) != c)
@@ -1394,8 +1445,9 @@ void
 int
 #endif
 winchg()
-{	hitwinch++;
-	(void) signal(SIGWINCH, winchg);
+{
+    hitwinch++;
+    (void) signal(SIGWINCH, winchg);
 }
 #endif
 
