@@ -6,7 +6,7 @@
  *			University of Maryland
  *		R. Bond  12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
- *		$Revision: 7.1 $
+ *		$Revision: 7.2 $
  *
  */
 
@@ -126,7 +126,7 @@ struct range {
 #define FIX_ROW 1
 #define FIX_COL 2
 
-/* stores type of operation this cell will preform */
+/* stores type of operation this cell will perform */
 struct enode {
     int op;
     union {
@@ -141,6 +141,20 @@ struct enode {
     } e;
 };
 
+/* Use this structure to save the last 'g' command */
+extern	struct go_save {
+	int g_type;
+	double g_n;
+	char *g_s;
+	int  g_row;
+	int  g_col;
+	int  g_lastrow;
+	int  g_lastcol;
+	int  strow;
+	int  stcol;
+	int  errsearch;
+} gs;
+
 /* op values */
 #define O_VAR 'v'
 #define O_CONST 'k'
@@ -149,65 +163,65 @@ struct enode {
 #define REDUCE 0200	/* Or'ed into OP if operand is a range */
 
 #define OP_BASE 256
-#define ACOS OP_BASE + 0
-#define ASIN OP_BASE + 1
-#define ATAN OP_BASE + 2
-#define CEIL OP_BASE + 3
-#define COS OP_BASE + 4
-#define EXP OP_BASE + 5 
-#define FABS OP_BASE + 6 
-#define FLOOR OP_BASE + 7
-#define HYPOT OP_BASE + 8
-#define LOG OP_BASE + 9
-#define LOG10 OP_BASE + 10
-#define POW OP_BASE + 11
-#define SIN OP_BASE + 12
-#define SQRT OP_BASE + 13
-#define TAN OP_BASE + 14
-#define DTR OP_BASE + 15
-#define RTD OP_BASE + 16
-#define MIN OP_BASE + 17
-#define MAX OP_BASE + 18
-#define RND OP_BASE + 19
-#define HOUR OP_BASE + 20
-#define MINUTE OP_BASE + 21
-#define SECOND OP_BASE + 22
-#define MONTH OP_BASE + 23
-#define DAY OP_BASE + 24
-#define YEAR OP_BASE + 25
-#define NOW OP_BASE + 26
-#define DATE OP_BASE + 27
-#define FMT OP_BASE + 28
-#define SUBSTR OP_BASE + 29
-#define STON OP_BASE + 30
-#define EQS OP_BASE + 31
-#define EXT OP_BASE + 32
-#define ELIST OP_BASE + 33	/* List of expressions */
-#define LMAX  OP_BASE + 34
-#define LMIN  OP_BASE + 35
-#define NVAL OP_BASE + 36
-#define SVAL OP_BASE + 37
-#define PV OP_BASE + 38
-#define FV OP_BASE + 39
-#define PMT OP_BASE + 40
-#define STINDEX OP_BASE + 41
-#define LOOKUP OP_BASE + 42
-#define ATAN2 OP_BASE + 43
-#define INDEX OP_BASE + 44
-#define DTS OP_BASE + 45
-#define TTS OP_BASE + 46
-#define ABS OP_BASE + 47 
-#define HLOOKUP OP_BASE + 48
-#define VLOOKUP OP_BASE + 49
-#define ROUND OP_BASE + 50
-#define IF OP_BASE + 51
-#define MYROW OP_BASE + 52
-#define MYCOL OP_BASE + 53
-#define COLTOA OP_BASE + 54
-#define UPPER OP_BASE + 55
-#define LOWER OP_BASE + 56
-#define CAPITAL OP_BASE + 57
-#define NUMITER	OP_BASE + 58
+#define ACOS (OP_BASE + 0)
+#define ASIN (OP_BASE + 1)
+#define ATAN (OP_BASE + 2)
+#define CEIL (OP_BASE + 3)
+#define COS (OP_BASE + 4)
+#define EXP (OP_BASE + 5 )
+#define FABS (OP_BASE + 6 )
+#define FLOOR (OP_BASE + 7)
+#define HYPOT (OP_BASE + 8)
+#define LOG (OP_BASE + 9)
+#define LOG10 (OP_BASE + 10)
+#define POW (OP_BASE + 11)
+#define SIN (OP_BASE + 12)
+#define SQRT (OP_BASE + 13)
+#define TAN (OP_BASE + 14)
+#define DTR (OP_BASE + 15)
+#define RTD (OP_BASE + 16)
+#define MIN (OP_BASE + 17)
+#define MAX (OP_BASE + 18)
+#define RND (OP_BASE + 19)
+#define HOUR (OP_BASE + 20)
+#define MINUTE (OP_BASE + 21)
+#define SECOND (OP_BASE + 22)
+#define MONTH (OP_BASE + 23)
+#define DAY (OP_BASE + 24)
+#define YEAR (OP_BASE + 25)
+#define NOW (OP_BASE + 26)
+#define DATE (OP_BASE + 27)
+#define FMT (OP_BASE + 28)
+#define SUBSTR (OP_BASE + 29)
+#define STON (OP_BASE + 30)
+#define EQS (OP_BASE + 31)
+#define EXT (OP_BASE + 32)
+#define ELIST (OP_BASE + 33	/* List of expressions */)
+#define LMAX  (OP_BASE + 34)
+#define LMIN  (OP_BASE + 35)
+#define NVAL (OP_BASE + 36)
+#define SVAL (OP_BASE + 37)
+#define PV (OP_BASE + 38)
+#define FV (OP_BASE + 39)
+#define PMT (OP_BASE + 40)
+#define STINDEX (OP_BASE + 41)
+#define LOOKUP (OP_BASE + 42)
+#define ATAN2 (OP_BASE + 43)
+#define INDEX (OP_BASE + 44)
+#define DTS (OP_BASE + 45)
+#define TTS (OP_BASE + 46)
+#define ABS (OP_BASE + 47 )
+#define HLOOKUP (OP_BASE + 48)
+#define VLOOKUP (OP_BASE + 49)
+#define ROUND (OP_BASE + 50)
+#define IF (OP_BASE + 51)
+#define MYROW (OP_BASE + 52)
+#define MYCOL (OP_BASE + 53)
+#define COLTOA (OP_BASE + 54)
+#define UPPER (OP_BASE + 55)
+#define LOWER (OP_BASE + 56)
+#define CAPITAL (OP_BASE + 57)
+#define NUMITER (OP_BASE + 58)
 
 /* flag values */
 #define is_valid     0001
@@ -254,9 +268,12 @@ extern	char curfile[];
 extern	int strow, stcol;
 extern	int currow, curcol;
 extern	int savedrow[26], savedcol[26];
+extern	int savedstrow[26], savedstcol[26];
 extern	int FullUpdate;
 extern	int maxrow, maxcol;
 extern	int maxrows, maxcols;	/* # cells currently allocated */
+extern	int rowsinrange;	/* Number of rows in target range of a move */
+extern	int colsinrange;	/* Number of cols in target range of a move */
 extern	int *fwidth;
 extern	int *precision;
 extern  int *realfmt;
@@ -267,6 +284,7 @@ extern	int linelim;
 extern	int changed;
 extern	struct ent *to_fix;
 extern	int showsc, showsr;
+extern	int cellassign;
 
 extern	FILE *openout();
 extern	char *coltoa(int col);
@@ -356,7 +374,8 @@ extern	void label(register struct ent *v, register char *s, int flushdir);
 extern	void let(struct ent *v, struct enode *e);
 extern	void list_range(FILE *f);
 extern	void lock_cells(struct ent *v1, struct ent *v2);
-extern	void moveto(int row, int col);
+extern	void moveto(int row, int col, int lastrow, int lastcol, int cornrow,
+    int corncol);
 extern	void num_search(double n, int errsearch);
 extern	void opencol(int cs, int numcol);
 extern	void printfile(char *fname, int r0, int c0, int rn, int cn);
@@ -409,8 +428,9 @@ extern	int showtop;
 extern	int loading;
 extern	int getrcqual;
 extern	int tbl_style;
-extern	int rndinfinity;
+extern	int rndtoeven;
 extern	char *progname;
+extern	int numeric_field;
 extern	int craction;
 extern	int rowlimit;
 extern	int collimit;
