@@ -1,7 +1,7 @@
 /*	SC	A Spreadsheet Calculator
  *
  *	One line vi emulation
- *	$Revision: 7.1 $
+ *	$Revision: 7.2 $
  */
 
 #include <sys/types.h>
@@ -15,9 +15,6 @@
 
 #include <signal.h>
 #include <curses.h>
-
-extern char	*strchr();
-
 #include <stdio.h>
 #include <ctype.h>
 #include "sc.h"
@@ -29,7 +26,6 @@ static void back_hist();
 static int  back_line();
 static int  back_word();
 static void back_space();
-static void change_cmd();
 static void col_0();
 static void cr_line();
 static void delete_cmd();
@@ -89,62 +85,64 @@ write_line(int c)
 {
     if (mode == EDIT_MODE) {
 	switch(c) {
-	case (ctl('h')):	linelim = back_line();		break;
-	case (ctl('m')):  cr_line();			break;
-	case ESC:	stop_edit();			break;
-	case '+':	for_hist();			break;
-	case '-':	back_hist();			break;
-	case '$':	last_col();			break;
-	case '.':	dotcmd();			break;
-	case '/':	search_mode();			break;
-	case '0':	col_0();			break;
-	case 'D':	u_save(c);del_to_end();		break;
-	case 'I':	u_save(c);col_0();insert_mode();break;
-	case 'R':	replace_mode();			break;
-	case 'X':	u_save(c); back_space();	break;
-	case 'a':	u_save(c); append_line();	break;
-	case 'A':	u_save(c);last_col();append_line();	break;
-	case 'b':	linelim = back_word();		break;
-	case 'c':	u_save(c); change_cmd();	break;
-	case 'd':	u_save(c); delete_cmd();	break;
-	case 'f':	linelim = find_char();		break;
-	case 'h':	linelim = back_line();		break;
-	case 'i':	u_save(c); insert_mode();	break;
-	case 'j':	for_hist();			break;
-	case 'k':	back_hist();			break;
+	case (ctl('h')):	linelim = back_line();			break;
+	case (ctl('m')):	cr_line();				break;
+	case ESC:	stop_edit();					break;
+	case '+':	for_hist();					break;
+	case '-':	back_hist();					break;
+	case '$':	last_col();					break;
+	case '.':	dotcmd();					break;
+	case '/':	search_mode();					break;
+	case '0':	col_0();					break;
+	case 'C':	u_save(c); del_to_end(); append_line();		break;
+	case 'D':	u_save(c); del_to_end();			break;
+	case 'I':	u_save(c); col_0(); insert_mode();		break;
+	case 'R':	replace_mode();					break;
+	case 'X':	u_save(c); back_space();			break;
+	case 'a':	u_save(c); append_line();			break;
+	case 'A':	u_save(c); last_col(); append_line();		break;
+	case 'b':	linelim = back_word();				break;
+	case 'c':	u_save(c); delete_cmd(); insert_mode();		break;
+	case 'd':	u_save(c); delete_cmd();			break;
+	case 'f':	linelim = find_char();				break;
+	case 'h':	linelim = back_line();				break;
+	case 'i':	u_save(c); insert_mode();			break;
+	case 'j':	for_hist();					break;
+	case 'k':	back_hist();					break;
 	case ' ':
-	case 'l':	linelim = for_line(0);		break;
-	case 'n':	search_again();			break;
-	case 'q':	stop_edit();			break;
-	case 'r':	u_save(c); rep_char();		break;
-	case 't':	linelim = to_char();		break;
-	case 'u':	restore_it();			break;
-	case 'w':	linelim = for_word(0);		break;
-	case 'x':	u_save(c); del_in_line();	break;
-	default:	break;
+	case 'l':	linelim = for_line(0);				break;
+	case 'n':	search_again();					break;
+	case 'q':	stop_edit();					break;
+	case 'r':	u_save(c); rep_char();				break;
+	case 's':	u_save(c); del_in_line(); insert_mode();	break;
+	case 't':	linelim = to_char();				break;
+	case 'u':	restore_it();					break;
+	case 'w':	linelim = for_word(0);				break;
+	case 'x':	u_save(c); del_in_line();			break;
+	default:							break;
 	}
     } else if (mode == INSERT_MODE) { 
 	savedot(c);
 	switch(c) {
-	case (ctl('h')):	back_space();			break;
-	case (ctl('m')):  cr_line();			break;
-	case ESC:	edit_mode();			break;
-	default:	ins_in_line(c);			break;
+	case (ctl('h')):	back_space();				break;
+	case (ctl('m')):	cr_line();				break;
+	case ESC:		edit_mode();				break;
+	default:		ins_in_line(c);				break;
 	}
     } else if (mode == SEARCH_MODE) {
 	switch(c) {
-	case (ctl('h')):	back_space();			break;
-	case (ctl('m')):  search_hist();			break;
-	case ESC:	edit_mode();			break;
-	default:	ins_in_line(c);			break;
+	case (ctl('h')):	back_space();				break;
+	case (ctl('m')):	search_hist();				break;
+	case ESC:		edit_mode();				break;
+	default:		ins_in_line(c);				break;
 	}
     } else if (mode == REP_MODE) {
 	savedot(c);
 	switch(c) {
-	case (ctl('h')):	back_space();			break;
-	case (ctl('m')):  cr_line();			break;
-	case ESC:	edit_mode();			break;
-	default:	replace_in_line(c);		break;
+	case (ctl('h')):	back_space();				break;
+	case (ctl('m')):	cr_line();				break;
+	case ESC:		edit_mode();				break;
+	default:		replace_in_line(c);			break;
 	}
     }
 }
@@ -292,6 +290,7 @@ static void
 stop_edit()
 {
     showrange = 0;
+    numeric_field = 0;
     linelim = -1;
     (void) move(1, 0);
     (void) clrtoeol();
@@ -368,8 +367,8 @@ back_word()
         while (cpos > 0 && line[cpos] == ' ')
 	    --cpos;
     } else if (cpos > 0 && (line[cpos-1] == ' ' 
-		     ||  istext(line[cpos]) && !istext(line[cpos-1])
-		     || !istext(line[cpos]) &&  istext(line[cpos-1]))) {
+		     || ( istext(line[cpos]) && !istext(line[cpos-1]))
+		     || (!istext(line[cpos]) &&  istext(line[cpos-1])))) {
 	/* Started on the first char of a word - back up to prev. word */
 	--cpos;
         while (cpos > 0 && line[cpos] == ' ')
@@ -519,13 +518,6 @@ delete_cmd()
     del_chars(cpos, linelim);
 }
 
-static void
-change_cmd()
-{
-    delete_cmd();
-    insert_mode();
-}
-
 static void
 del_chars(register int first, register int last)
 {
@@ -562,8 +554,10 @@ cr_line()
 	showrange = 0;
 	save_hist();
 	linelim = 0;
-	(void) yyparse ();
+	(void) yyparse();
 	linelim = -1;
+	if (cellassign) {
+	    cellassign = 0;
 	    switch (craction) {
 		case CRROWS:
 		    if ((rowlimit >= 0) && (currow >= rowlimit)) {
@@ -586,6 +580,7 @@ cr_line()
 		default:
 		    break;
 	    }
+	}
     }
     else	/* '\n' alone will put you into insert mode */
     {	*line = '\0';
